import 'dart:convert';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:pam_flutter/models/pam_message_model.dart';
import 'package:pam_flutter/pam_flutter.dart';
import 'package:pam_flutter/types.dart';
import 'package:flutter/services.dart' show rootBundle;

class PamNotification {
  PamNotification._();

  static String? _appPrefix;

  static Future<String> appPrefix() async {
    if (_appPrefix == null) {
      String raw = await rootBundle.loadString("pam-config.json");
      Map<String, dynamic> cfg = json.decode(raw);
      _appPrefix = cfg["app-prefix"];
    }
    return _appPrefix!;
  }

  static Future<void> pamInitial() async {
    FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin =
        FlutterLocalNotificationsPlugin();
    AndroidInitializationSettings initializationSettingsAndroid =
        AndroidInitializationSettings('app_icon');

    final IOSInitializationSettings initializationSettingsIOS =
        IOSInitializationSettings(
            onDidReceiveLocalNotification: _onDidReceiveLocalNotification);
    final InitializationSettings initializationSettings =
        InitializationSettings(
            android: initializationSettingsAndroid,
            iOS: initializationSettingsIOS);

    await flutterLocalNotificationsPlugin.initialize(initializationSettings,
        onSelectNotification: _selectNotification);
  }

  static Future _onDidReceiveLocalNotification(
      int id, String? title, String? body, String? payload) async {
    print("onDidReceiveLocalNotification");
  }

  static Future _selectNotification(String? payload) async {
    print("selectNotification");
  }

  static Future<bool> isPamMessageOpenedReceived(Map<String, dynamic>? message) async {
    if (message == null) {
      return false;
    }
    if (message["pam"] == null) {
      return false;
    }

    Map<String, dynamic> rawPam = json.decode(message["pam"] ?? "{}");
    PamMessageModel pam = PamMessageModel.fromJson(await appPrefix(), rawPam);

    Pam.dispatch(PamStandardCallback.on_message_opened, pam.toMap());
    return true;
  }

  static Future<bool> isPamMessageReceived(Map<String, dynamic>? message) async {
    if (message == null) {
      return false;
    }
    if (message["pam"] == null) {
      return false;
    }

    Map<String, dynamic> rawPam = json.decode(message["pam"] ?? "{}");
    PamMessageModel pam = PamMessageModel.fromJson(await appPrefix(), rawPam);

    Pam.dispatch(PamStandardCallback.on_message, pam.toMap());
    return true;
  }
}
